using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace DynoLib
{
	/// <summary>
	/// Summary description for UserControl1.
	/// </summary>
	public class DynoGraph : BasicGraph
	{
		#region Member Data
		public enum Indices {IND_RPM,IND_HP,IND_TQ,IND_MIX,IND_TPS,IND_LASTRPM, IND_MAX};
		/*public static int IND_RPM = 0;
		public static int IND_HP = 1;
		public static int IND_TQ = 2;
		public static int IND_MIX = 3;
		public static int IND_TPS = 4;
		public static int IND_LASTRPM = 5;*/

		private static Font mainFont = new Font("Verdana", 9f);
		private static Font boldFont = new Font("Verdana", 9f, FontStyle.Bold);
		private static float PADDING = 5f;
		private PointF mousePos;
		private Timer delayTimer;
		private bool showbox;
		private DynoPopupData popData;

		private static string powerUnits;

		//the IND_* int that represents the graph thats currently being pointed at
		private int currGraph;

		private MixtureGraphConstraints mixData;
		#endregion

		#region Constructor
		public DynoGraph()
		{
			InitializeComponent();

			this.MouseMove +=new MouseEventHandler(DynoGraph_MouseMove);
			mousePos = new Point(0,0);

			delayTimer = new Timer();
			delayTimer.Interval = 200;
			delayTimer.Tick += new EventHandler(delayTimer_Tick);

			popData = new DynoPopupData();
			mixData = new MixtureGraphConstraints();

			powerUnits = "hp & ft.lbs";

			scaledata.NumberOfScales = new Point(8,15);
		}
		#endregion

		#region Utilities
		public void addDynoRun(DynoRun run)
		{
			if(run == null)
				return;

			run.SortData();

			int ind = findRunIndex(run.Name);
			if(ind > -1) //replace it
			{
				GraphData oldrun = (GraphData)graphs[ind];
				run.GraphColor = oldrun.GraphColor;
				graphs.Insert(ind,run);
				graphs.RemoveAt(ind+1);
			}
			else //normal add
			{
				graphs.Add(run);
				base.pickColor(run,graphs.Count-1);
			}
			ResetGraphLimits();

			this.OnRunsChanged(run,true);
		}
		public void addDynoRun(DynoRun run, int index, bool overwrite)
		{
			if(run == null || index<0 || index>=graphs.Count)
				return;

			run.SortData();

			GraphData oldrun = findRun(run.Name);

			graphs.Insert(index,run);
			base.pickColor(run,index);

			if(overwrite)
				graphs.RemoveAt(index+1);

			if(oldrun != null)
				graphs.Remove(oldrun);

			ResetGraphLimits();

			this.OnRunsChanged(run,true);
		}

		private float floor(float num, int numzeros)
		{
			float zs = 1f;
			for(int i = 0; i<numzeros; i++)
				zs*=10f;

			num = num/zs;
			num = (float)((int)num);

			return num*zs;
		}

		public void removeDynoRun(GraphData run)
		{
			graphs.Remove(run);
			this.OnRunsChanged(run,false);
			ResetGraphLimits();
		}

		public void ResetGraphLimits()
		{
			if(graphs.Count == 0)
			{
				scaledata.Max = new PointF(3000f,100f);
				scaledata.Min = new PointF(0f,0f);
				mixData.MixtureMax = 16f;
				mixData.MixtureMin = 10f;
				return;
			}

			scaledata.Max = new PointF(0f,0f);
			scaledata.Min = new PointF(int.MaxValue,int.MaxValue);
			mixData.MixtureMax = 0;
			mixData.MixtureMin = int.MaxValue;

			foreach( GraphData run in graphs )
			{

				float ym = Math.Max(scaledata.Max.Y, run.getMax(new int[]{1}) + scaledata.Scale.Y/2f);
				float xm = Math.Max(scaledata.Max.X, run.getMax(new int[]{0}) + scaledata.Scale.X/2f);
				scaledata.Max = new PointF(xm,ym);

				ym = floor(Math.Min(scaledata.Min.Y, run.getMin(new int[]{1})),1);
				xm = floor(Math.Min(scaledata.Min.X, run.getMin(new int[]{0})),2);
				scaledata.Min = new PointF(xm,ym);

				mixData.MixtureMax = Math.Max(mixData.MixtureMax, run.getMax(new int[]{3}));
				mixData.MixtureMin = Math.Min(mixData.MixtureMin,run.getMin(new int[]{3}));

				xm = (float)Math.Ceiling((double)(scaledata.Max.X- scaledata.Min.X)/(double)scaledata.NumberOfScales.X);
				ym = (float)Math.Ceiling((double)(scaledata.Max.Y- scaledata.Min.Y)/(double)scaledata.NumberOfScales.Y);
				scaledata.Scale = new PointF(xm,ym);
			}
		}

		/// <summary>
		/// Translates the points for one graph from real-world values to 
		/// graph coords.
		/// </summary>
		/// <param name="arr">the run data</param>
		/// <param name="datatype">the x index corresponding to the desired graph</param>
		/// <returns>an array of graph coords</returns>
		private PointF[] TranslatePoints(ArrayList arr, int datatype)
		{
			float ychunk1 = (float)this.ClientRectangle.Bottom - mixData.GraphHeight - scaledata.Indent.Y;
			float ychunk2 = ((float)this.ClientRectangle.Height - scaledata.Indent.Y - mixData.GraphHeight) / scaledata.Max.Y;
			float xchunk = ((float)this.ClientRectangle.Width - scaledata.Indent.X)/(scaledata.Max.X - scaledata.Min.X);
			
			PointF[] pts = new PointF[arr.Count];

			int pti = 0;
			for(int i = 0; i<arr.Count; i++)
			{
				if(checkIgnoreRules((GraphItem)arr[i]))
				{
					pts[pti] = new PointF(scaledata.Indent.X +  xchunk * (((GraphItem)arr[i]).Data[0]- scaledata.Min.X),
						ychunk1 - ychunk2*((GraphItem)arr[i]).Data[datatype]);
					pti++;
				}
			}
			for(;pti<pts.Length; pti++)
				pts[pti] = pts[pti-1];
			return pts;
		}

		private PointF[] TranslateMixturePoints(ArrayList arr, int datatype)
		{
			float ychunk1 = (float)this.ClientRectangle.Bottom - scaledata.Indent.Y;
			float ychunk2 = (mixData.GraphHeight - mixData.GraphPadding) / (mixData.MixtureMax - mixData.MixtureMin);
			float xchunk = ((float)this.ClientRectangle.Width - scaledata.Indent.X)/(scaledata.Max.X - scaledata.Min.X);
			
			PointF[] pts = new PointF[arr.Count];

			int pti = 0;
			for(int i = 0; i<arr.Count; i++)
			{
				if(checkIgnoreRules((GraphItem)arr[i]))
				{
					pts[pti] = new PointF(scaledata.Indent.X +  xchunk * (((GraphItem)arr[i]).Data[0]- scaledata.Min.X),
						ychunk1 - ychunk2*(((GraphItem)arr[i]).Data[datatype]-mixData.MixtureMin));
					pti++;
				}
			}
			for(;pti<pts.Length; pti++)
				pts[pti] = pts[pti-1];

			return pts;
		}

		/// <summary>
		/// Translate a real data point into a point on the graph.
		/// </summary>
		/// <param name="x"></param>
		/// <param name="y"></param>
		/// <returns></returns>
		private PointF TranslatePoint(float x, float y)
		{
			float xx = (this.scaledata.Indent.X + ((float)this.ClientRectangle.Width - scaledata.Indent.X) * (x- scaledata.Min.X) / (scaledata.Max.X- scaledata.Min.X));
			float yy = (float)this.ClientRectangle.Bottom-mixData.GraphHeight-(scaledata.Indent.Y + ((float)this.ClientRectangle.Height - scaledata.Indent.Y-mixData.GraphHeight) * y / scaledata.Max.Y);

			return new PointF(xx,yy);
		}

		private PointF TranslateMixturePoint(float x, float y)
		{
			float xx = (this.scaledata.Indent.X + ((float)this.ClientRectangle.Width - scaledata.Indent.X) * (x- scaledata.Min.X) / (scaledata.Max.X - scaledata.Min.X));
			float yy = (float)this.ClientRectangle.Bottom-(scaledata.Indent.Y + (mixData.GraphHeight - mixData.GraphPadding) * (y-mixData.MixtureMin) / (mixData.MixtureMax - mixData.MixtureMin));

			return new PointF(xx,yy);
		}

		/// <summary>
		/// Translate mouse coordinates into real world coords.
		/// </summary>
		/// <returns></returns>
		private PointF TranslatePoint()
		{
			float xx = ((mousePos.X-this.scaledata.Indent.X)*(scaledata.Max.X- scaledata.Min.X)/
				((float)ClientRectangle.Width-scaledata.Indent.X))+ scaledata.Min.X;
			float yy = ((mousePos.Y - (float)ClientRectangle.Bottom + scaledata.Indent.Y + 
				mixData.GraphHeight)*scaledata.Max.Y/(scaledata.Indent.Y+mixData.GraphHeight-
				(float)ClientRectangle.Height));

			if(yy<0f)
				yy =  mixData.MixtureMin + (mixData.MixtureMax - mixData.MixtureMin) *
					(mousePos.Y - (float)ClientRectangle.Bottom + scaledata.Indent.Y)/
					(mixData.GraphHeight - mixData.GraphPadding);

			return new PointF(xx,yy);
		}

		private float EstimateRunValue(PointF mouse, DynoRun run, int gindex, Point runi)
		{
			if(runi == Point.Empty || mouse == PointF.Empty || run == null || 
				gindex < 0 || gindex >= run.RunData.Count)
				return -1f;

			float fakeY = run.GetItem(runi.X).Data[gindex];
			if(runi.Y > -1)
			{
				float slope = (run.GetItem(runi.Y).Data[gindex] - fakeY)/
					(run.GetItem(runi.Y).Data[0] - run.GetItem(runi.X).Data[0]);

				fakeY = (mouse.X-run.GetItem(runi.X).Data[0]) * slope + fakeY;
			}

			return fakeY;
		}

		private float PointingAtGraph(PointF mouse, DynoRun run, int gindex, Point runi)
		{
			float fakeY = EstimateRunValue(mouse,run,gindex,runi);

			PointF gcoords;
			if(gindex == (int)Indices.IND_MIX)
				gcoords = TranslateMixturePoint(0,fakeY);
			else
				gcoords = TranslatePoint(0,fakeY);

			if(mousePos.Y >= gcoords.Y-PADDING && mousePos.Y <= gcoords.Y+PADDING)
					return fakeY;
			return -1f;
		}
		#endregion

		#region Painting!!

		protected override void OnPaintBackground(PaintEventArgs pe){}
		protected override void OnPaint( PaintEventArgs pe )
		{
			Graphics g = null;

			if(dbuffer.CanDoubleBuffer())
				g = dbuffer.g;
			else
				g = pe.Graphics;

			bool showmixture = false;
			foreach(GraphData run in graphs)
				showmixture = showmixture || run.GetGraphShowing((int)Indices.IND_MIX);

			if(showmixture)
				mixData.GraphHeightViewable = mixData.GraphHeightOld;
			else
				mixData.GraphHeightViewable = 0f;

			g.FillRectangle(new SolidBrush(Color.Black), pe.ClipRectangle.X, pe.ClipRectangle.Y, pe.ClipRectangle.Width, pe.ClipRectangle.Height);

			g.SmoothingMode = SmoothingMode.AntiAlias;
			DrawXAxis(g);
			DrawYAxis(g);
			DrawCurves(g);
			DrawInfoBox(g,popData.getStrings());

			if(dbuffer.CanDoubleBuffer())
				dbuffer.Render(pe.Graphics);
		}

		#region Axis Darwing
		private void DrawXAxis(Graphics g)
		{
			Pen pen = new Pen(Color.LightSlateGray, 1);
			SolidBrush brush = new SolidBrush(Color.LightSlateGray);

			float ybott = (float)ClientRectangle.Bottom - scaledata.Indent.Y - mixData.GraphHeight;
			float xcoord = (float)ClientRectangle.Left + scaledata.Indent.X;
			float ycoord = ybott+mixData.GraphHeight;

			float xPos = scaledata.Min.X;
			float scale = ((float)this.ClientRectangle.Width - scaledata.Indent.X) * 
							scaledata.Scale.X / (scaledata.Max.X-scaledata.Min.X);

			g.DrawLine( pen, xcoord, ybott, (float)ClientRectangle.Right, ybott);
			g.DrawLine( pen, xcoord, ycoord, (float)ClientRectangle.Right, ycoord);
			
			ycoord = ybott+ mixData.GraphPadding;
			if(mixData.GraphHeightViewable > 0f)
				g.DrawLine( pen, xcoord, ycoord, (float)ClientRectangle.Right, ycoord);

			for (float i = scaledata.Indent.X; i < (float)this.ClientRectangle.Width; i += scale )
			{
				g.DrawString(xPos.ToString("0"), mainFont, brush, i, ((float)this.ClientRectangle.Bottom - scaledata.Indent.Y + (float)mainFont.Height/3f));

				xcoord = (float)(this.ClientRectangle.Left) + i;
				g.DrawLine(pen, xcoord, ybott, xcoord, (float)(this.ClientRectangle.Top));

				if(mixData.GraphHeightViewable > 0f)
					g.DrawLine(pen, xcoord, (float)ClientRectangle.Bottom - scaledata.Indent.Y, xcoord, ybott + mixData.GraphPadding);

				xPos += scaledata.Scale.X;
			}

			SizeF sz = g.MeasureString("RPM",boldFont);
			g.DrawString("RPM",boldFont,brush,(float)this.ClientRectangle.Width/2f - sz.Width/2f + scaledata.Indent.X,
						(float)ClientRectangle.Bottom - sz.Height - 3f);

			pen.Dispose();
			brush.Dispose();
		}
		
		private void DrawYAxis(Graphics g)
		{
		
			Pen pen = new Pen(Color.LightSlateGray, 1);
			SolidBrush brush = new SolidBrush(Color.LightSlateGray); 

			float xleft = (float)ClientRectangle.Left + scaledata.Indent.X;
			float yPos = 0;
			float scale  = ((float)ClientRectangle.Height - scaledata.Indent.Y - mixData.GraphHeight) * scaledata.Scale.Y / scaledata.Max.Y;

			g.DrawLine( pen, xleft, (float)ClientRectangle.Top, xleft, (float)ClientRectangle.Bottom  - scaledata.Indent.Y - mixData.GraphHeight );

			for (float i = scaledata.Indent.Y + mixData.GraphHeight; i < (float)ClientRectangle.Height; i += scale)
			{				
				float f = g.MeasureString(yPos.ToString(),mainFont).Width;
				g.DrawString(yPos.ToString("0"), mainFont, brush, xleft - (f + 1), this.ClientRectangle.Bottom - i - mainFont.GetHeight(g)/2f);
				g.DrawLine(pen, xleft, (float)(this.ClientRectangle.Bottom) - i, (float)(this.ClientRectangle.Right), (float)(this.ClientRectangle.Bottom) - i);

				yPos += scaledata.Scale.Y;
			}	

			float afrscale = mixData.YScale;
			scale = (mixData.GraphHeight - mixData.GraphPadding) / mixData.NumScaleLines;
			yPos = mixData.MixtureMin;
			for (float i = scaledata.Indent.Y; i <= mixData.GraphHeight +scaledata.Indent.Y- mixData.GraphPadding; i += scale)//* afrscale / (mixData.MixtureMax - mixData.MixtureMin))
			{				
				float f = g.MeasureString(yPos.ToString("0.0"),mainFont).Width;
				g.DrawString(yPos.ToString("0.0"), mainFont, brush, xleft - (f + 1), this.ClientRectangle.Bottom - i - mainFont.GetHeight(g)/2f);
				g.DrawLine(pen, xleft, (float)(this.ClientRectangle.Bottom) - i, (float)(this.ClientRectangle.Right), (float)(this.ClientRectangle.Bottom) - i);

				yPos += afrscale;
			}

		//	StringFormat drawFormat = new StringFormat(StringFormatFlags.DirectionVertical);
			string power = "Power ("+ powerUnits + ")";
			SizeF sz = g.MeasureString(power,boldFont);//,100, drawFormat);
			g.RotateTransform(-90f);
			g.DrawString(power,boldFont,brush,
				(-(float)ClientRectangle.Height+mixData.GraphHeight + 
				scaledata.Indent.Y)/2f - sz.Width/2f,2f);

			if(mixData.GraphHeightViewable > 0f)
			{
				sz = g.MeasureString(mixData.Units,boldFont);
				g.DrawString(mixData.Units,boldFont,brush,
					-(float)ClientRectangle.Height+(mixData.GraphHeight-mixData.GraphPadding)/2f +
					scaledata.Indent.Y - sz.Width/2f,2f);
			}

			g.ResetTransform();
	
			pen.Dispose();
			brush.Dispose();
		//*/	
		}
		#endregion

		#region line/curve drawing
		private void DrawCurves(Graphics g)
		{
			
			foreach(DynoRun run in this.graphs)
			{
				Pen p = null;
				if(!run.Visible)
					continue;
				if(run.GetGraphShowing((int)Indices.IND_HP))
				{
					p = new Pen(run.GetColor((int)Indices.IND_HP),2F);
					DrawCurve(g,p,run,1);
					p.Dispose();
				}

				if(run.GetGraphShowing((int)Indices.IND_TQ))
				{
					p = new Pen(run.GetColor((int)Indices.IND_TQ),2F);
					DrawCurve(g,p,run,2);
					p.Dispose();
				}

				if(run.GetGraphShowing((int)Indices.IND_MIX))
				{
					p = new Pen(run.GetColor((int)Indices.IND_MIX),2F);
					DrawCurve(g,p,run,3);
					p.Dispose();
				}
			}
			
		}

		private void DrawCurve(Graphics g, Pen p, DynoRun run, int ind)
		{
			int max = run.RunData.Count;
			PointF[] points;
			if(ind == (int)Indices.IND_MIX)
				points = TranslateMixturePoints(run.RunData,ind);
			else
				points = TranslatePoints(run.RunData,ind);

			if(LineMode == LINEMODE_LINE)
				DrawLine(g,p,points);
			else
				g.DrawCurve(p,points);
		}

		private void DrawLine(Graphics g, Pen p, PointF[] points)
		{
			PointF lastp = points[0];//TranslatePoint(0F,0F);

			foreach(PointF po in points)
			{
				g.DrawLine(p,lastp,po);
				lastp = po;
			}
		}
		#endregion

		#region Box Drawing
		private SizeF MeasureStrings(Graphics g,string[] s)
		{
			SizeF sz = new SizeF(0f,0f);

			foreach(string str in s)
			{
				SizeF tmp = g.MeasureString(str,boldFont);
				sz.Height += tmp.Height;
				sz.Width = Math.Max(sz.Width,tmp.Width);
			}

			return sz;
		}

		private void DrawInfoBox(Graphics g, string[] s)
		{
			if(!showbox)
				return;

		//	s = mousePos.X + ", " + mousePos.Y;
			SizeF sz = MeasureStrings(g,s);

			float w = sz.Width+10f;
			float h = sz.Height+10f;
			float xp = mousePos.X-w-2f;
			float yp = mousePos.Y-h-2f;

			if(xp < 1f)
				xp = 1f;
			if(yp < 1f)
				yp = 1f;
			RectangleF rect = new RectangleF(xp,yp,w,h);

			SolidBrush b = new SolidBrush(Color.Black);
			g.FillRectangle(b,rect);

			Pen p = new Pen(Color.LightSlateGray,2F);
			g.DrawRectangle(p,xp,yp,w,h);

			b.Dispose();
			b = new SolidBrush(Color.LightSlateGray);
			SolidBrush sel = new SolidBrush(Color.Red);

			xp += 5f;
			yp += 5f;
			float he = boldFont.GetHeight(g);


			for(int i = 0; i<s.Length; i++)
			{
				if(this.currGraph == i-1)
					g.DrawString(s[i],boldFont,sel,xp,yp);
				else
					g.DrawString(s[i],mainFont,b,xp,yp);
				yp += he;
			}

			b.Dispose();
			sel.Dispose();
			p.Dispose();
		}
		#endregion

		#endregion

		#region Events
		private void DynoGraph_MouseMove(object sender, MouseEventArgs e)
		{
			mousePos.X = e.X;
			mousePos.Y = e.Y;

			if(showbox)
			{
				showbox = false;
				Refresh();
			}
			delayTimer.Stop();
			delayTimer.Start();
		}
		private void delayTimer_Tick(object sender, EventArgs e)
		{
			delayTimer.Stop();
			
			bool newshow = false;
			PointF pos = TranslatePoint();
			foreach( DynoRun run in graphs )
			{
				Point p = run.BinarySearch(pos.X);

				for(int i = 1; i< 4/*run.GetItem(0).DataLength*/; i++)
				{
					if(PointingAtGraph(pos, run, i, p)>0f)
					{
						popData.RPM = pos.X;
						popData.RunName = run.Name;
						popData.HorsePower = EstimateRunValue(pos, run, 1, p);
						popData.Torque = EstimateRunValue(pos, run, 2, p);
						popData.AFR = EstimateRunValue(pos, run, 3, p);

						newshow = true;
						currGraph = i;
						break;
					}
				}

				if(newshow)
					break;
			}

			if(showbox != newshow || newshow)
			{
				showbox = newshow;
				Refresh();
			}
			showbox = newshow;
		}
		#endregion

		#region Useless Crap
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// DynoGraph
			// 
			this.Name = "DynoGraph";

		}
		#endregion
	}	
}
