using System;
using System.Windows.Forms;
using System.Drawing;
using System.Collections;

namespace DynoLib
{
	/// <summary>
	/// A class representing the underlying data that represents some graph. 
	/// </summary>
	/// The meat of the class is in the RunData array which is a 2D float array
	/// holding all of the graph's data. It is organized as follows: 
	///  - There can be multiple curves with each GraphData object
	///  - Each is given an index in the x component of the array 
	///    - x=0 represents the x axis data
	///    - x>0 represents the y data for each graph
	///  - the y component selects a sample from the data
	///  
	///  Additionally, a color for this run is stored in memory. An 
	///  alpha channel value can be set for each run, thereby enabling
	///  each graph more or less transparent. 
	public class GraphData
	{
		//private float[,] runData; //actual data from the run...
		private ArrayList allData;
		private ArrayList runData;
		private string name;//could be the file name?
		private Color graphColor; //one for hp,tq,afr??
		private byte[] alphaNums;
		private bool[] showGraph;
		private int interval;
		private bool isVisible;


		#region Properties
		public string Name
		{
			get{ return name; }
			set{ name = value; }
		}

		public bool Visible
		{
			get{return isVisible;}
			set{isVisible = value;}
		}

		public Color GraphColor
		{
			get{ return graphColor; }
			set{ graphColor = value; }
		}

		public int Interval
		{
			get{return interval;}
			set{interval = value;}
		}

		/// <summary>
		/// \returns an ArrayList containing ALL of the GraphItem objects
		/// that may need to be displayed at some point
		/// </summary>
		public ArrayList AllData//float[,] RunData
		{
			get{ return allData; }
			set
			{ 
				allData = value; 
				if(allData != null && allData.Count>0)
				{
					alphaNums = new byte[((GraphItem)allData[0]).DataLength];
					showGraph = new bool[alphaNums.Length];
					for(int i = 0; i<alphaNums.Length; i++)
					{
						alphaNums[i] = 255;
						showGraph[i] = true;
					}
				}//end if
			}//end set
		}

		/// <summary>
		/// \returns an ArrayList containing only the GraphItem objects
		/// that need to be displayed
		/// </summary>
		public ArrayList RunData//float[,] RunData
		{
			get{ return runData; }
			set
			{ 
				runData = value; 
				if(allData == null)
					AllData = value;
			}//end set
		}
		#endregion

		public GraphData(string uniqueName, Color c)
		{
			name = uniqueName;
			graphColor = c;
			Visible = true;
		}

		#region Gets/Sets
		public byte GetAlphaChannel(int ind)
		{
			if(!isInBounds(ind))
				return 255;

			return alphaNums[ind-1];
		}
		public void SetAlphaChannel(byte alpha, int ind)
		{
			if(!isInBounds(ind))
				return;

			alphaNums[ind-1] = alpha;
		}

		public Color GetColor(int ind)
		{
			if(!isInBounds(ind))
				return Color.Empty;

			return Color.FromArgb((int)alphaNums[ind-1],GraphColor.R,GraphColor.G,GraphColor.B);
		}

		public bool GetGraphShowing(int ind)
		{
			if(!isInBounds(ind))
				return false;

			return showGraph[ind-1];
		}
		public void SetGraphShowing(bool show, int ind)
		{
			if(!isInBounds(ind))
				return;

			showGraph[ind-1] = show;
		}

		public GraphItem GetItem(int index)
		{
			if(index<runData.Count && index>-1)
				return (GraphItem)runData[index];
			return null;
		}
		#endregion

		#region utilities

		private bool isInBounds(int ind)
		{
			return ind > 0 && ind <= alphaNums.Length;
		}
		public float getMax(int[] index)
		{
			if(runData == null)
				return 0f;
			float max = 0.0f;

			foreach(int i in index)
			{
				if(i > GetItem(0).DataLength || i<0)
					continue;
				for(int j = 0; j< runData.Count; j++)
					if(max<GetItem(j).Data[i])
						max = GetItem(j).Data[i];
			}

			return max;
		}

		public float getMin(int[] index)
		{
			if(runData == null)
				return 0f;
			float min = float.MaxValue;

			foreach(int i in index)
			{
				if(i > GetItem(0).DataLength || i<0)
					continue;
				for(int j = 0; j< runData.Count; j++)
					if(min>((GraphItem)runData[j]).Data[i])
						min = ((GraphItem)runData[j]).Data[i];
			}

			return min;
		}

		public Point BinarySearch(float key) 
		{
			return BinarySearch(runData,key);
		}

		/// <summary>
		/// Binary Searches the key data in a run. (key such as rpm)
		/// </summary>
		/// <param name="run"></param>
		/// <param name="key"></param>
		/// <returns>A point indicating the indicies in the run array which
		/// surround the given key. If the key is directly matched, a point 
		/// with the index in the X coordinate and a -1 in the Y coordinate
		/// will be returned. </returns>
		public static Point BinarySearch(ArrayList run, float key)  
		{
			int l = 0;
			int r = run.Count-1;

			while (r >= l)
			{
				int m = (l + r) / 2;

				GraphItem item = (GraphItem)run[m];

				if (key == item.Data[0])
					return new Point(m,-1); //indicating exact match. very unlikely.
				if(m > 0 && key < item.Data[0] && key > ((GraphItem)run[m-1]).Data[0])
					return new Point(m-1,m);
				if(m < run.Count-1 && key > item.Data[0] && key < ((GraphItem)run[m+1]).Data[0])
					return new Point(m,m+1);

				if (key < item.Data[0])
					r = m - 1;
				else
					l = m + 1;          
			}
			return Point.Empty;
		}

		public void SortData()
		{
			if(runData == null)
				return;

			runData.Sort();

			/*
			int[] tagArray = new int[runData.GetUpperBound(1)+1];

			for(int i = 0; i<tagArray.Length; i++)
				tagArray[i] = i;

			// Initialize the comparer and sort
			RectangularComparer myComparer = new RectangularComparer(runData);
			Array.Sort(tagArray, myComparer);

			float[,] newdata = new float[GetItem(0).DataLength+1,tagArray.Length];
			for (int i = 0; i < tagArray.Length; i++)
				for(int j = 0; j<= GetItem(0).DataLength; j++)
					newdata[j,i] = runData[j,tagArray[i]];

			runData = null;
			runData = newdata;
			*/
		}

		private class RectangularComparer: IComparer
		{
			// maintain a reference to the 2-dimensional array being sorted
			private float[,] sortArray;

			// constructor initializes the sortArray reference
			public RectangularComparer(float[,] theArray)
			{
				sortArray = theArray;
			}

			public int Compare(object x, object y)
			{
				// x and y are integer row numbers into the sortArray
				int i1 = (int)x;
				int i2 = (int)y;

				// compare the items in the sortArray
				return sortArray[0,i1].CompareTo(sortArray[0,i2]);
			}
		}
		#endregion
	}
}
