/*
 * BGraphLib - Map Graph Control
 * Created for BRE
 * 
 * Copyright (c) 2005, Brian Rogers
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this 
 * list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 * 
 * Neither the name of the organization nor the names of its contributors may 
 * be used to endorse or promote products derived from this software without 
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace BGraphLib
{
	/// <summary>
	/// Summary description for MapPlot.
	/// </summary>
	public class MapGraphControl : System.Windows.Forms.UserControl
	{

		#region Properties
		/// <summary>
		/// Specifies the 2D array of map values
		/// </summary>
		public double[,] Map
		{
			get
			{
				return gridMap;
			}
			set
			{
				gridMap = value;
			//	cols = gridMap.GetUpperBound(0);
			//	rows = gridMap.GetLowerBound(1);
				maxY = (int)(FindMaxValue(gridMap) + 0.5) + yScale;
			}
		}
		private double[,] gridMap;
		

		/// <summary>
		/// Specifies the array of RPM values on the x-axis
		/// </summary>
		public int[] RpmList
		{
			get
			{
				return rpmList;
			}
			set
			{
				rpmList = value;

				maxX = FindMaxValue(rpmList);
				maxX += 1000 - maxX % 1000;		// round up to nearest 1000
			}
		}
		private int[] rpmList;
		

		/// <summary>
		/// Specifies the array of MAP values
		/// </summary>
		public string[] PressureList
		{
			get
			{
				return pressureList;
			}
			set
			{
				pressureList = value;
			}
		}
		private string[] pressureList;
		

		public delegate void UpdateHandler(object sender, GraphNodeDragEventArgs e);
		public event UpdateHandler NodeMoved;

		#endregion

		#region Private Variables
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private int maxY;
		private int maxX;

		private int cols;
		private int rows;

		private int xScale = 1000, yScale = 2;
		private int xAxisIndent = 25, yAxisIndent = 25;

		private DoubleBuffer doubleBuffer;

		private GraphNode[,] graphNodes;

		private Graphics g;

		private GraphNode dragNode = null;
		private GraphNode selectedNode = null;

		private bool drag = false;
		#endregion

		
		public MapGraphControl()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			this.Load += new EventHandler(MapGraphControl_Load);
			this.Resize += new EventHandler(MapGraphControl_Resize);
			this.MouseMove += new MouseEventHandler(MapGraphControl_MouseMove);
			this.MouseDown += new MouseEventHandler(MapGraphControl_MouseDown);
			this.MouseUp += new MouseEventHandler(MapGraphControl_MouseUp);

			doubleBuffer = new DoubleBuffer();
			graphNodes = new GraphNode[50,50];
		}
		public MapGraphControl(int c, int r)
		{
			// This call is required by the Windows.Forms Form Designer.
			gridMap = new double[c,r];
			rpmList = new int[r];
			pressureList = new string[c];
			InitializeComponent();
			cols = c;
			rows = r;
			

			this.Load += new EventHandler(MapGraphControl_Load);
			this.Resize += new EventHandler(MapGraphControl_Resize);
			this.MouseMove += new MouseEventHandler(MapGraphControl_MouseMove);
			this.MouseDown += new MouseEventHandler(MapGraphControl_MouseDown);
			this.MouseUp += new MouseEventHandler(MapGraphControl_MouseUp);

			doubleBuffer = new DoubleBuffer();
			graphNodes = new GraphNode[c,r];
		}
		public void setSize(int c, int r)
		{
			cols = c;
			rows = r;
			rpmList = new int[r];
			gridMap = new double[r,c];
			graphNodes = new GraphNode[r,c];
			pressureList = new string[c];
		}
		public void setupHeaders()
		{
			maxY = (int)(FindMaxValue(gridMap) + 0.5) + yScale;
			maxX = FindMaxValue(rpmList);
			maxX += 1000;// - maxX % 1000;		// round up to nearest 1000
		}
		public void setRow(int row, int val)
		{
			rpmList[row] = val;
		}
		public void setCol(int col, string val)
		{
			pressureList[col] = val;
		}
		public void setValue(int col, int row, double val)
		{
//			MessageBox.Show(col.ToString() +", "+ row.ToString() + ": " + gridMap[row,col]);
			gridMap[row,col] = val;
			Refresh();
		}
		public double getValue(int col, int row)
		{
			return gridMap[row,col];
		}
		private void MapGraphControl_Load(object sender, EventArgs e)
		{
			 doubleBuffer.CreateDoubleBuffer(null, this.ClientRectangle.Width, this.ClientRectangle.Height);
		//	MessageBox.Show(doubleBuffer.CanDoubleBuffer().ToString());
		}

		protected override void OnPaint( PaintEventArgs pe )
		{
			if(doubleBuffer.CanDoubleBuffer())
			{
				this.g = doubleBuffer.g;
				this.g.FillRectangle(new SolidBrush(Color.Black), pe.ClipRectangle.X, pe.ClipRectangle.Y, pe.ClipRectangle.Width, pe.ClipRectangle.Height);

			}
			else
			{
				this.g = pe.Graphics;
				this.g.FillRectangle(new SolidBrush(Color.Black), pe.ClipRectangle.X, pe.ClipRectangle.Y, pe.ClipRectangle.Width, pe.ClipRectangle.Height);
			}

	//		try
	//		{
				DrawXAxis();
				DrawYAxis();
				PlotPoints();
	//		}
	//		catch{}
			if(drag)
				ShowLabel(dragNode.LocalY.ToString("N2"), dragNode.WorldX, dragNode.WorldY);

			if(doubleBuffer.CanDoubleBuffer())
				doubleBuffer.Render(pe.Graphics);

		}

		protected override void OnPaintBackground(PaintEventArgs pevent)
		{
		}

		private void PlotPoints()
		{
			SolidBrush brush = new SolidBrush(Color.Red);
			Pen pen = new Pen( Color.Red, 1 );
			pen.Width = 2; 

			selectedNode = null;

			for(int i = 0; i <= Map.GetUpperBound(1); i++)
			{
				int j = 0;
				GraphNode prevNode = null;

				// set line colors
				int green = (255 - (int)(255 * gridMap[j,i] / gridMap[0,gridMap.GetUpperBound(1)]))%255;
				if(green<0)
					green*=-1;
				pen.Color = Color.FromArgb(255, green, 0);
				brush.Color = Color.FromArgb(255, green, 0);
		//		if(Map != null)
		//		{
					for(j = 0; j <= Map.GetUpperBound(0); j++)
					{
						// translate local to world coords
						int y = (int)((double)this.ClientRectangle.Height - this.ClientRectangle.Height * (gridMap[j,i] / maxY)) - yAxisIndent;
						int x = (int)((this.ClientRectangle.Width) * ((float)rpmList[j] / maxX)) + xAxisIndent;	
				
						if(graphNodes[j,i] == null)
						{
							graphNodes[j,i] = new GraphNode(x, y, rpmList[j], gridMap[j,i], j, i);
						}
						else if(!drag)
						{
							graphNodes[j,i].WorldX = x;
							graphNodes[j,i].WorldY = y;						
							graphNodes[j,i].LocalY = gridMap[j,i];
						}

						GraphNode curNode = graphNodes[j,i];

						// draw some stuff
						g.FillRectangle(brush, curNode.WorldX, curNode.WorldY - 2, 5, 5);

						if(prevNode != null)
							g.DrawLine(pen, prevNode.WorldX, prevNode.WorldY, curNode.WorldX, curNode.WorldY);

						if(graphNodes[j,i].IsSelected)
						{
							selectedNode = graphNodes[j,i];
						
							g.DrawRectangle(new Pen(Color.White), graphNodes[j,i].WorldX - 3, graphNodes[j,i].WorldY - 5, 10, 10);	
						}
				
						prevNode = curNode;
					}//end inner for

				if(pressureList!=null)
					if(pressureList.Length > i )
						if(pressureList[i] !=null)
							g.DrawString(pressureList[i], new Font("Verdana", 7, FontStyle.Bold), new SolidBrush(Color.White), graphNodes[j-1,i].WorldX+5, graphNodes[j-1,i].WorldY-6);
		//					ShowLabel(pressureList[i], graphNodes[j-1,i].WorldX+10, graphNodes[j-1,i].WorldY+13);
				

			}

			if(selectedNode != null)
				ShowLabel(selectedNode.LocalY.ToString("N2"), selectedNode.WorldX, selectedNode.WorldY);
			pen.Dispose();
			brush.Dispose();
		}

		private void DrawXAxis()
		{
			Pen pen = new Pen(Color.LightSlateGray, 1);
			g.DrawLine( pen, ClientRectangle.Left + xAxisIndent, ClientRectangle.Bottom - yAxisIndent, ClientRectangle.Right, ClientRectangle.Bottom - yAxisIndent );
			
			Font theAxisFont = new Font("Verdana", 9);
			SolidBrush brush = new SolidBrush(Color.LightSlateGray);

			int xPos = 0;

			for (float i = xAxisIndent; i < this.ClientRectangle.Width; i += this.ClientRectangle.Width / (maxX / xScale))
			{
				g.DrawString(xPos.ToString(), theAxisFont, brush, i, (float)(this.ClientRectangle.Bottom - (xAxisIndent - theAxisFont.Height/3)));
				g.DrawLine(pen, (float)(this.ClientRectangle.Left) + i, (float)(this.ClientRectangle.Bottom - yAxisIndent), (float)(this.ClientRectangle.Left) +  i, (float)(this.ClientRectangle.Top));

				xPos += xScale;
			}		

			pen.Dispose();
			brush.Dispose();
			theAxisFont.Dispose();
		}
		
		private void DrawYAxis()
		{
			Pen pen = new Pen(Color.LightSlateGray, 1);
			g.DrawLine( pen, ClientRectangle.Left + xAxisIndent, ClientRectangle.Top, ClientRectangle.Left + xAxisIndent, ClientRectangle.Bottom  - yAxisIndent );

			Font theAxisFont = new Font("Verdana", 9);
			SolidBrush brush = new SolidBrush(Color.LightSlateGray); 
			
			int yPos = 0;

			for (float i = yAxisIndent; i < this.ClientRectangle.Height; i += this.ClientRectangle.Height / (maxY / yScale))
			{				
				g.DrawString(yPos.ToString(), theAxisFont, brush, this.ClientRectangle.Left + xAxisIndent - (theAxisFont.Height + 6), this.ClientRectangle.Bottom - i);
				g.DrawLine(pen, (float)(this.ClientRectangle.Left + xAxisIndent), (float)(this.ClientRectangle.Bottom) - i, (float)(this.ClientRectangle.Right), (float)(this.ClientRectangle.Bottom) - i);

				yPos += yScale;
			}	
	
			pen.Dispose();
			theAxisFont.Dispose();
			brush.Dispose();
		}

		private void ShowLabel(string label, int x, int y)
		{
			
			g.FillRectangle(new SolidBrush(Color.Black), x - 5, y - 20, 35, 15);
			g.DrawString(label, new Font("Verdana", 7, FontStyle.Bold), new SolidBrush(Color.White), x - 5, y - 20);
		}

		private double FindMaxValue(double[,] array)
		{
			double result = 0.0;

			for(int x = 0; x <= array.GetUpperBound(0); x++)
			{
				for(int y = 0; y <= array.GetUpperBound(1); y++)
				{
					if(array[x,y] > result)
						result = array[x,y];
				}
			}

			return result;
		}

		private int FindMaxValue(int[] array)
		{
			int result = 0;

			for(int i = 0; i <= array.GetUpperBound(0); i++)
			{
				if(array[i] > result)
					result = array[i];
			}

			return result;
		}

		

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion

		private void MapGraphControl_Resize(object sender, EventArgs e)
		{
			doubleBuffer.CreateDoubleBuffer(this.CreateGraphics(), this.ClientRectangle.Width, this.ClientRectangle.Height);
			Invalidate(); // Force a repaint after has been resized 
		}

		private void MapGraphControl_MouseMove(object sender, MouseEventArgs e)
		{
			if(drag)
			{
				double newY = -(double)(((e.Y + yAxisIndent - this.ClientRectangle.Height) * maxY)) / this.ClientRectangle.Height;

				if(newY < 0)
				{
					dragNode.WorldY = this.ClientRectangle.Height - yAxisIndent;
					dragNode.LocalY = 0;
				}
				else if(e.Y < 0)
				{
					dragNode.WorldY = 0;				
					dragNode.LocalY = maxY;
				}
				else
				{
					dragNode.WorldY = e.Y;				
					dragNode.LocalY = newY;
				}
				
				this.Refresh();
			}
//			else if(e.Button == MouseButtons.None)
//			{
//				for(int i = 0; i <= Map.GetUpperBound(1); i++)
//				{
//					for(int j = 0; j <= Map.GetLowerBound(0); j++)
//					{
//						if(graphNodes[j,i] == null)
//							continue;
//
//						GraphNode p = graphNodes[j,i];
//						Rectangle r = new Rectangle(p.WorldX - 5, p.WorldY - 6, 10, 12);
//						
//						if(r.Contains(e.X, e.Y))
//							p.IsSelected = true;
//						else
//							p.IsSelected = false;
//					}
//				}
//
//				this.Refresh();
//			}
		}

		private void MapGraphControl_MouseDown(object sender, MouseEventArgs e)
		{
			for(int i = 0; i <= Map.GetUpperBound(1); i++)
			{
				for(int j = 0; j <= Map.GetUpperBound(0); j++)
				{
					if(graphNodes[j,i] == null)
						continue;

					GraphNode p = graphNodes[j,i];						
					Rectangle r = new Rectangle(p.WorldX - 5, p.WorldY - 6, 10, 12);
						
					if(r.Contains(e.X, e.Y))
					{
						dragNode = p;
						drag = true;
						Refresh();
						return;
					}
				}
			}
		}

		private void MapGraphControl_MouseUp(object sender, MouseEventArgs e)
		{
			if(!drag)
				return;


			maxY = (int)(FindMaxValue(gridMap) + 0.5) + yScale;			 

			double newY = -(double)(((e.Y + yAxisIndent - this.ClientRectangle.Height) * maxY)) / this.ClientRectangle.Height;


			if(newY < 0)
			{
				newY = 0;
				dragNode.WorldY = this.ClientRectangle.Height - yAxisIndent;
				dragNode.LocalY = 0;
			}
			else if(e.Y < 0)
			{
				dragNode.WorldY = 0;				
				dragNode.LocalY = maxY;
			}
			else
			{
				dragNode.WorldY = e.Y;				
				dragNode.LocalY = newY;
			}

			// throw event
			try
			{
				gridMap[dragNode.IndexX,dragNode.IndexY] = newY;
				NodeMoved(this, new GraphNodeDragEventArgs(newY, dragNode.IndexY, dragNode.IndexX));
			}
			catch{}


			drag = false;
			this.Refresh();
		}
	}
}
