#region Licence...
//-----------------------------------------------------------------------------
// Date:	17/10/04    Time: 2:33p
// Module:	csscript.cs
// Classes:	CSExecutor
//			ExecuteOptions
//
// This module contains the definition of the CSExecutor class. Which implements 
// compiling C# code and executing 'Main' method of compiled assembly
//
// Written by Oleg Shilo (oshilo@gmail.com)
// Copyright (c) 2004. All rights reserved.
//
// Redistribution and use of this code in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
// 1. Redistributions of source code must retain the above copyright notice, 
//    this list of conditions and the following disclaimer. 
// 2. Neither the name of an author nor the names of the contributors may be used 
//    to endorse or promote products derived from this software without specific 
//    prior written permission.
// 3. This code may be used in compiled form in any way you desire. This
//	  file may be redistributed unmodified by any means PROVIDING it is 
//    not sold for profit without the authors written consent, and 
//    providing that this notice and the authors name is included. 
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//	Caution: Bugs are expected!
//----------------------------------------------
#endregion 

using System;
using System.IO;
using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System.Reflection;
using System.Security.Policy;
using System.Collections;
using System.Threading;
using System.Text;

namespace csscript
{
	/// <summary>
	/// CSExecutor is an class that implements execution of *.cs files.
	/// </summary>
	class CSExecutor
	{
		#region Public interface...
		/// <summary>
		/// Force caught exceptions to be rethrown.
		/// </summary>
		public bool Rethrow
		{
			get {return rethrow;}
			set {rethrow = value;}
		}


		/// <summary>
		/// Checks/returns if compiled C# script file (ScriptName + "c") available and valid.
		/// </summary>
		public static string GetAvailableAssembly(string scripFileName) 
		{
			string retval = null;
			string asmFileName = scripFileName + "c";
			if (File.Exists(asmFileName) && File.Exists(scripFileName))
			{
				FileInfo scriptFile = new FileInfo(scripFileName);
				FileInfo asmFile = new FileInfo(asmFileName);
				if( asmFile.LastWriteTime == scriptFile.LastWriteTime && 
					asmFile.LastWriteTimeUtc == scriptFile.LastWriteTimeUtc)
				{
					retval = asmFileName;
				}
			}
			return retval;
		} 

		

		/// <summary>
		/// Compiles C# script file into assembly.
		/// </summary>
		/// <param name="scriptFile"></param>
		/// <param name="assemblyFile"></param>
		/// <param name="debugBuild"></param>
		/// <returns>Returns [0] compiler results, [1] assembly path</returns>
		public object[] Compile(string scriptFile, string assemblyFile, bool debugBuild) 
		{
			if (assemblyFile != null)
				options["forceOutputAssembly"] = assemblyFile;
			else
				options["forceOutputAssembly"] = Path.GetTempFileName();
			if (debugBuild)
				options["DBG"] = true;
			return Compile(scriptFile);
		}
		
		#endregion 

		#region Class nested declarations...
		/// <summary>
		/// Application specific version of Hashtable
		/// </summary>
		private class ExecuteOptions : Hashtable
		{
			public ExecuteOptions()
			{
				this["processFile"] = true;
				this["scriptFileName"] = "";
			}
			public bool IsSet(string name)
			{
				return this[name] != null;
			}
			public bool GetBool(string name)
			{
				return this[name] != null ? (bool)this[name] : false; //default is false
			}
		}
		#endregion 

		#region Class data...
		/// <summary>
		/// Callback to print application messages to appropriate output.
		/// </summary>
		static PrintDelegate print;
		/// <summary>
		/// Container for paresed command line parguments
		/// </summary>
		ExecuteOptions options;
		/// <summary>
		/// Flag to force to rethrow critical exceptions
		/// </summary>
		bool rethrow;
		#endregion 

		#region Class methods...
		/// <summary>
		/// Constructor
		/// </summary>
		public CSExecutor()
		{
			rethrow = false;
			options = new ExecuteOptions();
		}

//		private Assembly Compile(string scriptFileName, Assembly a) 
//		{
//
//			CompilerResults results = (CompilerResults)Compile(scriptFileName)[0];
//			return results.CompiledAssembly;
//		//	return assemblyFileName;
//		}
//
//		private string Compile(string scriptFileName, string s) 
//		{
//			string assemblyFileName = (string)Compile(scriptFileName)[1];
//			return assemblyFileName;
//		}

		/// <summary>
		/// Compiles C# script file.
		/// </summary>
		private object[] Compile(string scriptFileName) 
		{
			bool generateExe = options.GetBool("buildExecutable");
			bool localAssembliesUsed = false;
			string scriptDir = Path.GetDirectoryName(scriptFileName);
			string assemblyFileName = "";

			//parse source file in order to find all referenced assemblies
			//ASSUMPTION: assembly name is the same as namespace + ".dll"
			//if script doesn't follow this assumption user will need to 
			//specify assemblies explicitly  
			ScriptParser parser = new ScriptParser(scriptFileName);

			if (parser.apartmentState != System.Threading.ApartmentState.Unknown)
				System.Threading.Thread.CurrentThread.ApartmentState = parser.apartmentState;

			ICodeCompiler compiler = (new CSharpCodeProvider()).CreateCompiler();
			CompilerParameters compileParams = new CompilerParameters();
         	

			compileParams.IncludeDebugInformation = options.GetBool("DBG");
			compileParams.GenerateExecutable = false;//generateExe;
			compileParams.GenerateInMemory = true;//!generateExe;
			
			//some assemblies were referenced from command line
//			if (options.IsSet("refAssemblies"))
//			{
//				foreach(string asmName in (string[])options["refAssemblies"])
//				{
//					string asmLocation = Path.Combine(scriptDir, asmName);
//					if (File.Exists(asmLocation))
//					{
//						compileParams.ReferencedAssemblies.Add(asmLocation);
//						localAssembliesUsed = true;
//					}
//				}
//			}
	
		
			AssemblyResolver.ignoreFileName = Path.GetFileNameWithoutExtension(scriptFileName) + ".dll";

			foreach (string nmSpace in parser.ReferencedNamespaces) 
			{
				//find local and global assemblies assuming assembly name is the same as a namespace
				foreach (string asmLocation in AssemblyResolver.FindAssembly(nmSpace, scriptDir)) 
				{
					compileParams.ReferencedAssemblies.Add(asmLocation);

					if (!localAssembliesUsed)
						localAssembliesUsed = (Path.GetDirectoryName(asmLocation) == scriptDir);
				}
			}
			
			foreach (string asmName in parser.ReferencedAssemblies) //some assemblies were referenced from code
			{
				string asmLocation = Path.Combine(scriptDir, asmName);
				if (File.Exists(asmLocation))
				{
				//	print("exists: " + asmLocation);
					compileParams.ReferencedAssemblies.Add(asmLocation);
					localAssembliesUsed = true;
				}
				else
					throw new Exception("Referenced assembly doesnt exist: " + asmLocation);
			}

//			if (options.IsSet("forceOutputAssembly"))
//			{
				assemblyFileName = (string) options["forceOutputAssembly"];
//			}
//			else
//			{
//				if (generateExe)
//					assemblyFileName = Path.Combine(scriptDir, Path.GetFileNameWithoutExtension(scriptFileName) + ".exe");
//				else if (options.GetBool("useCompiled") || localAssembliesUsed)
//					if (options.GetBool("DLLExtension") )
//						assemblyFileName = Path.Combine(scriptDir, Path.GetFileNameWithoutExtension(scriptFileName) + ".dll");
//					else
//						assemblyFileName = Path.Combine(scriptDir, scriptFileName + "c");
//				else
//					assemblyFileName = Path.GetTempFileName();
//			}

			compileParams.OutputAssembly = assemblyFileName;

	//		if (generateExe && options.GetBool("buildWinExecutable"))
	//			compileParams.CompilerOptions = "/target:winexe";

			if (File.Exists(assemblyFileName))
				File.Delete(assemblyFileName);
			
			CompilerResults results = compiler.CompileAssemblyFromFileBatch(compileParams, parser.FilesToCompile);
			if (results.Errors.Count != 0) 
			{
				StringBuilder compileErr = new StringBuilder();
				foreach (CompilerError err in results.Errors) 
				{
					compileErr.Append(err.ToString());
				}
				throw new Exception(compileErr.ToString());				
			}
			else
			{
				parser.DeleteImportedFiles();

				if (!options.GetBool("DBG")) //pdb file might be needed for a debugger
				{
					string pdbFile = Path.Combine(Path.GetDirectoryName(assemblyFileName), Path.GetFileNameWithoutExtension(assemblyFileName)+".pdb");	
					if (File.Exists(pdbFile))
						File.Delete(pdbFile);
				}

				FileInfo scriptFile = new FileInfo(scriptFileName);
				FileInfo asmFile = new FileInfo(assemblyFileName);
				if (scriptFile!= null && asmFile != null)
				{
					asmFile.LastWriteTime = scriptFile.LastWriteTime; 
					asmFile.LastWriteTimeUtc = scriptFile.LastWriteTimeUtc;
				}
			}
			object[] o = new object[2];
			o[0] = results;
			o[1] = assemblyFileName;
			return o;
		}
	
		/// <summary>
		/// Prints Help info.
		/// </summary>
		private void ShowHelp()
		{
		} 
		/// <summary>
		/// Show sample C# scrip file.
		/// </summary>
		private void ShowSample()
		{
		}
		#endregion
	}

}			
