/*! \page loggerusage Datalogger and Log Analyzer Extension

\section Introduction

The Logger and LogAnalyzer objects are now based on defined rules. They 
now only are a framework that contains most of the functionality that
any datalogger or log analyzer would need. For example, the Logger class
provides a way to send and recieve data, and output that data to the user.
It does NOT provide the functionality specific to one ecu. Thats where the 
rules come in. 

A Rule is essentialy an interface with which the Logger can notify your code
that some task needs to be done. In the case of the Logger, rules allow the 
programmer to have complete control over what is sent to the ecu, 
manipulations on the data recieved back from the ecu, and the data output to 
the screen and to the log file. In the case of the LogAnalyzer, the programmer
has control over the data being displayed and analyzed through rules.

\section extdl Extending the Datalogger

In order to extend the BRE data logger You need to do a few things:

 - create a new script to load into BRE
 - create a class that extends bre20.Rule that is either in the same
   script file or referenced by the previously mentioned script file.
 - make the Rule class create a bre20.LoggerKey object (usually) so that 
   the calculations you do will be output to the screen and/or to the 
   csv file.
 - add the new Rule to the datalogger via \c child.addLoggerRule() 

There is an example below in the \ref ruleexamp "Examples" section.

\section specfun Special Functions

There are a few special functions that must be set by some Rule that is running 
in the Logger. The values kept by these functions are automatically added to the 
csv file so the analyzer can properly analyze. These values are NOT added to the 
output grid. If you want them in the grid you must create a LoggerKey for them. The special
functions in the Logger class are:

\code
	//These functions must be used at each iteration of the datalogger.

	//self explanatory
	void setAfr(float afr)
	void setMapIndex(int index)
	void setCurrentColumn(int x)
	void setCurrentFuelRow(int fy)
	void setCurrentIgnitionRow(int iy)

	//This sets a cells weight based on the interpolation.
	//index [0,0] is the current cell, index [1,0]
	//is the east cell, [0,1] is the south cell, etc.
	//The weights should always add up to 1.
	void setCellWeight(int col, int row, float value)

\endcode

The same functions are provided in the AutoEdit class where the values are stored.
They are only in the Logger class for convenience. The get counterparts to these 
functions are in the AutoEdit class as well.  

\section newo2 Adding a New O2 sensor Definition

With the Rule framework it is easy to add a new O2 sensor definition to the Logger.
You need to first create a script which catches the OnDataloggerCreated event:

\code
	public object ScriptMain(params object[] args)
	{
		romWind = (child)args[1];
		romWind.subscribeToEvent("OnDataloggerCreated", "BensScripts.NewO2Def.log_Opened");
		return null;
	}
\endcode

Then define a function that adds an object of type AfrRule to the Logger:

\code
	public object log_Opened(params object[] args)
	{
		if(args[2] == null)
			return null;
		((Logger)args[2]).addO2Type(new NewO2Rule("My New O2"));
		return null;
	}
\endcode

You obviously need to create the object that extends AfrRule. But the key to 
it is the executeRule function: 

\code
	public bool executeRule(Form frm)
	{
		Logger lo = ((Logger)frm);
		//is it our o2 type? if not return false
		if(index != lo.getO2Type()) return false;
		
		//get value from the hash table
		Integer val = lo.getLoggerData().getHashtable()["o2afr"];

		//if its null, return false
		if(val == null) return false;

		//do calc stuff here:
		float afr = (float)(val.Value)/2.0f+9.0f;

		//set the value for the output
		afrKey->Value = afr.ToString("0.00");

		//set the afr for the rest of the Logger
		lo.setAfr(afr);

		return true;
	}
\endcode

The full example is down in the \ref o2examp "Examples" section.

\section dooev Logger Order of Events...

\subsection dlinstant ...On Instantiation

 - The OnDataloggerOpened event is fired and the Logger object is NULL. 
   Rules must be added at this time. To add a Rule at this time,
   subscribe to the the OnLoggerOpened event in the the child (rom window) 
   object. If you add them before this (i.e. only once in ScriptMain or 
   something) then they will only work in the Logger one time. After the
   Logger form is closed, the rules are deleted.

 - The Logger Object is instantiated.

 - The LoggerData object is set up which builds the command list 
   from the ini file. 

 - The setupRule function is called at each Rule. You can do any necessary 
   processing here. You will recieve the Logger form(cast it as a Logger 
   object). At this time you should add your LoggerKeys 
   to the LoggerData object. Keep a reference to these objects so you
   can update them. Example:
   \code
	LoggerKey rpmKey = new LoggerKey("Rpm",true,true);
	datalogger.getLoggerData().addLoggerKey(rpmKey);
	//...
	rpmKey.Value = rpm.ToString();
   \endcode

 - The OnDataloggerCreated event is fired. Now you can do any processing on a
   valid Logger object. 

 - Add new o2 rules at this time. To do this you must subscribe to the
   OnDataloggerCreated event.

 - You can add Gauges to the digital dash at this time as well. 

 - The grid on the log form is setup. The array of LoggerKey objects is passed to it
   to allow it to set the grid size and add the labels to the grid.

\subsection hitstart ...When The User Clicks Start

 - The OnDataLoggerStart event is fired

 - The timer is started...

\subsection running ...When The Datalogger Is Running

 - The command list is sent to the ecu, and the results are stored in a hash
   table in the LoggerData object (available through \c Logger.getLoggerData()). 

 - \c executeRule() is executed on each Rule. You will recieve the Logger (cast it
   as a Logger object) through which you can get the results data from 
   the hash table. 

 - The OnDataLoggerRun event is fired.

 - The auto tune stuff is run if necessary.

\section aooev LogAnalyzer Order of Events...

\subsection aninstant ...On Instantiation

 - The OnLogAnalyzerOpened event is fired and the Logger object is NULL. 
   Rules must be added at this time. To add a Rule at this time,
   subscribe to the the OnLogAnalyzerOpened event in the the child (rom window) 
   object. If you add them before this (i.e. only once in ScriptMain or 
   something) then they will only work in the Analyzer one time. After the
   Logger form is closed, the rules are deleted.

 - The LogAnalyzer object is instantiated

 - The headers in the file are read, put into a hash table, then the setupRule()
   function of your rule is called. It is important to not that every time the 
   user applies different settings, the log nalyzer re reads the input file. So,
   every time a use changes some setting, the setupRule() function is called again.

 - The data is then read in. The LogAnalyzer reads in a line of data, does its 
   processing and then calls the executeRule() function in each Rule.

\section examp Examples

\subsection o2examp New O2 Sensor Definition

\code
//This adds a new o2 sensor definition to BRE

using System;
using System.Windows.Forms;

using bre20;
//css_reference ..\bre20.exe;

namespace BensScripts
{
	public class NewO2Def
	{
		public static object AppliesTo(){return "all";}
		public static object ScriptTitle(){return "New O2 Def by Ben Ogle";}
		//args[1] = child window of type bre20.child
		public object ScriptMain(params object[] args)
		{
			romWind = (child)args[1];
			romWind.subscribeToEvent("OnDataloggerCreated", "BensScripts.NewO2Def.log_Opened");
			return null;
		}
		public object log_Opened(params object[] args)
		{
			if(args[2] == null)
				return null;

			((Logger)args[2]).addO2Type(new NewO2Rule("My New O2"));

			return null;
		}
	}//end class

	public class NewO2Rule: AfrRule
	{
		//stuff we can use:
		//afrKey
		//index
		public NewO2Rule(string name){base(name);}

		public bool executeRule(Form frm)
		{
			Logger lo = ((Logger)frm);
			//is it our o2 type? if not return false
			if(index != lo.getO2Type()) return false;
			
			//get value from the hash table
			Integer val = lo.getLoggerData().getHashtable()["o2afr"];

			//if its null, return false
			if(val == null) return false;

			//do calc stuff here:
			float afr = (float)(val.Value)/2.0f+9.0f;

			//set the value for the output
			afrKey->Value = afr.ToString("0.00");

			//set the afr for the rest of the Logger
			lo.setAfr(afr);

			return true;
		}
	}
}//end namespace
\endcode

\subsection ruleexamp General Rule Example
\code
//This adds a Datlogger definition definition to BRE

using System;
using System.Windows.Forms;
using System.Collections;

using bre20;
//css_reference ..\bre20.exe;

namespace BensScripts
{
	public class Vt0Logger
	{
		child romWind;
		public static object AppliesTo(){return "pr3:pw0";}
		public static object ScriptTitle(){return "Obd0 Vtec Logger Defn by Ben Ogle";}
		//args[1] = child window of type bre20.child
		public object ScriptMain(params object[] args)
		{
			romWind = (child)args[1];
			//subscribe to the event
			romWind.subscribeToEvent("OnDataloggerOpened", "BensScripts.Vt0Logger.log_Opened");
			return null;
		}
		public object log_Opened(params object[] args)
		{
			//when the logger is opened, then we add our rule
			romWind.addLoggerRule(new Vt0LoggerDef());
			return null;
		}
	}//end class

	public class Vt0LoggerDef: Rule
	{
		Logger log;
		//save a reference to our key
		LoggerKey rpm;
		public Vt0LoggerDef(){}
		public override void setupRule(Form frm)
		{
			log = ((Logger)frm);

			//here we set up the rpm LoggerKey
			rpm = new LoggerKey("Rpm",true,true);
			//add it to the logger
			log.getLoggerData().addLoggerKey(rpm);
		}
		public override bool executeRule(Form frm)
		{	
			//get the results values from the hash table
			//the names/labels/keys are from loggerCmd.ini
			Hashtable hash = log.getLoggerData().getHashtable();
			Integer val = (Integer)hash["rpm low"];
			int tmp = val.Value + 256*(((Integer)hash["rpm high"]).Value);

			//set the value for the output
			rpm.Value = tmp.ToString();
			return true;
		}
		//when a log file is being replayed this gets called. If you create
		//LoggerKeys which are not put in the csv file or gauges, you will
		//need to update those objects in this method. BRE automatically 
		//matches the values in the csv file with their LoggerKey object;
		//those values will be put in the output grid for you. But BRE does
		//not update gauges. 
		public override bool executeReplay(string[] keys, Hashtable values)
		{
			return true;
		}
		//this is what we are modifying
		public override string modifiedFields(){return "rpm";}
	}
}//end namespace
\endcode
*/