
#pragma once
using namespace System;
using namespace System::Windows::Forms;

//#include "FormWithRules.h"
#include "LoggerData.h"

namespace bre20
{
	/*!
	This is the interface for all Rules used by the datalogger and
	the log analyzer.

	A rule is an object that enables the programmer to easily extend
	both the dataloger and the log analyzer. You would be able to define
	a rule that, say, converted the raw RPM into a real world RPM value.
	Or if a new command is added to the logging code (and the list that
	the logger uses to send commands to the ecu), you can define a new 
	rule that properly converts the raw value to an acceptable value.

	This is an abstract class so you obviously must define the functions
	contained within in order to extend it.
	*/
	public __gc class Rule
	{
	public:
		Rule(){};
		/*!
		Called the first time the FormWithRules form
		knows about your rule.

		\param frm The form you will be given to grab values from or act on
		*/
		virtual void setupRule(Form* frm)=0;

		/*!
		Called after all the execute functions have been called.
		*/
		virtual void finishRule(Form* frm){};

		/*!
		This function is called when a rule needs to be 
		executed. 

		In the context of the Datalogger, it will be called each 
		time the logger gets data back from an ECU. You are passed
		the Datalogger and will then be able to modify the data
		from the ecu before its displayed.

		Additionally if your rule needs to deal with a gauge you 
		must update that gauge in this function.

		\param frm The form you will be given to grab values from or act on
		\returns true if it executed properly, false if otherwise
		*/
		virtual bool executeRule(Form* frm)=0;

		/*!
		This function is called when a rule needs to be 
		updated. 

		In the context of the Datalogger, it will be called each 
		time the DatalogOptions dialog is opened and closed. You are passed
		the Datalogger and will update the rule's display as necessary
		(such as with a change in the units).

		Additionally if your rule needs to deal with a gauge you 
		must update that gauge in this function.
		*/
		virtual bool updateRule(Form* frm)=0;

		/*!
		This function is the same as executeRule() but happens when a log is 
		being replayed. It is necessary for updating gauges when replaying. The
		text based values are automatically put into their respective places when 
		replaying.

		\param keys The labels from the log file.
		\param values has values from the file mapped to each key. 
		\returns true if executed properly
		*/
		virtual bool executeReplay(String* keys[], Hashtable* values)=0;

		/*!
		A string that explains what fields you are modifying. see
		\ref rulefield "the rule field explanation".
		
		If you only modify one field just return the name of that 
		field. If concatination is necessary, the fields will be 
		separated by colons ( \c: ). i.e.
		\code
		return "rpm:index:tps";
		\endcode

		\returns A string that explains what fields you are modifying.
		*/
		virtual String* modifiedFields()=0;
	};

	///See \ref loggerusage "Datalogger Extension" for usage guidelines.
public __gc class AfrRule: public Rule
	{
	protected:
		int index;
		String* name;
		///just use this supplied LoggerKey object to set afr value.
		LoggerKey* afrKey;
	public:
		AfrRule(String* sensorname):Rule(){name = sensorname;}

		int getIndex(){return index;}
		void setIndex(int i){index = i;}

		String* getName(){return name;}
		void setName(String* sensorname){name = sensorname;}

		void setAfrKey(LoggerKey* afrkey){afrKey = afrkey;}

		//necessary stuff
		virtual String* modifiedFields(){return "afr";}
		virtual void setupRule(Form* frm){};
		virtual bool updateRule(Form* frm){return true;};
		virtual bool executeReplay(String* keys[], Hashtable* vals){return true;};
		virtual bool executeRule(Form* frm)=0; 
	};
}

/*! \page rulefield Rule Fields Explained
	
	A rule field is a text label that corresponds to some certain 
	datalog value. You can think of it as a key for the key value 
	pair of a name and a datalogged value. 

	You can find out what rule fields are supported for your ecu 
	by looking at the RomDefs file in the BRE directory. It has a 
	rule field/key that corresponds to each ram location. 
*/