#pragma once

using namespace System;
using namespace System::Collections;

#include "bytechange.h"
#include "romrepeventargs.h"

namespace RomRep
{
	///Most likely, this Object
	///will need to know about the HondaEcu object that owns the Map 
	///object (which the Map object does not know). This is easy to get 
	///around by having the HondaEcu object as a member of the derived 
	///class (ben, have the hondaEcu class or some script create these!).
	public __gc class HeaderCalculator
	{
	protected:
		Object* m_map; //this will be a Map object!!
		Object* m_hondaecu; //this will be a HondaEcu object!!
	public:
		HeaderCalculator(Object* map, Object* hondaecu);

		///Called when the client (a grid page or something) needs to know
		///whether some header can be edited.
		virtual bool canEditHeader(bool column, int columnOrRow){return false;};

		///This is used by the map at least initially when the map is setup by passing
		///a HEADER_EVENT_ALL int into the column or row to indicate it wants all headers.
		///This object should know about the map its being called on so a mechanism for
		///grabbing the header location(s) should be in place.
		virtual RomRep::MapHeaderChangeArgs* calculateHeaders(bool col, int columnOrRow)=0;

		///This will be called when a byte changes in the Rom. You can then 
		///decide to or not to update your state (if necessary) and return
		///a MapHeaderChangeArgs object. If the object you return is NOT null,
		///then the Map will pretty much blindly dump the string array contents 
		///into its header arrays and fire the appreoriate events for the UI.
		virtual  RomRep::MapHeaderChangeArgs* byteChangeNotify(ByteChange* bc){return NULL;};

		///These set functions should take the string, use a regex 
		///and try to extract the data. If possible, they will calculate
		///the byte(s) to be changed and do a send byte request to the 
		///owning map.
		virtual RomRep::ByteChange* setRowHeader(int row, String* val)[]{return NULL;};
		///see setRow Header.
		virtual RomRep::ByteChange* setColumnHeader(int col, String* val)[]{return NULL;};

		///Splits up the MapHeaderChangeArgs calls into setRowHeader and
		///setColumnHeader calls. This does not need to be overriden
		virtual void mapHeaderChanged(MapHeaderChangeArgs* args);

		///This is a reference array mostly for the interpolation. It should be 
		///some set of values that all share the same unit for these headers.
		///i.e. an array of rpm values (row headers) or load values (col headers) 
		///raw or otherwise 
		virtual float getReferenceArray() __gc[]=0;
	};
	public __gc class HeaderCalculatorPair
	{
	private:
		HeaderCalculator* rowCalculator;
		HeaderCalculator* columnCalculator;
	public:
		HeaderCalculatorPair(){}

		HeaderCalculator* GetRowCalculator(){return rowCalculator;}
		HeaderCalculator* GetColumnCalculator(){return columnCalculator;}

		void SetRowCalculator(HeaderCalculator* calculator){rowCalculator = calculator;}
		void SetColumnCalculator(HeaderCalculator* calculator){columnCalculator = calculator;}
	};
	//!
	//!Idea behind this: Have a base class that all other maps can be derived from.
	//!Data will be stored in its byte and will be returned as real world value.
	//!
	//!Concrete derived Maps will get passed an integer representation of the map (from
	//!the raw file) then will convert the map and store it in this class' 2D array.
	//!
	///The basics of the Map event structure and the way they are used are as follows:
	///- When something is changed in the GUI or in the HondaEcu object owning this map,
	///  this Map wil get one or more calls (such as the notifyByteChange function
	///  call from HondaEcu). The events are processed and may or may not generate more 
	///  events. Sounds like common sense, eh?
	///- Example: a byte changes in the ROM
	///  - byteChangeNotify is called with the change
	///  - the map checks to see if it cares about the byte that changed. 
	///    - if the byte is in the map, the new value in the map is calculated and 
	///      an OnRealMapChanged event is fired for some GUI object to catch.
	///    - this byte is passed to the HeaderCalculator objects. If its a header byte,
	///      the HeaderCalculator will return a valid MapHeaderChangeArgs which will
	///      be used to update the header array(s) and an OnMapHeaderChanged event
	///      will be fired.
	///- Example: user edits a header in the GUI
	///  - The GUI generates an event which calls mapHeaderChanged.
	///    - This calls all of the mapHeaderChanged functions in the HeaderCalculator objects
	///      - The new edited header is parsed by each HeaderCalculator object
	///      - The HeaderCalculator objects will use the owner Map to request byte changes from
	///        the HondaEcu object. If those requests are accepted, the byte is changed and 
	///        HondaEcu generates a byte change event which will be handled as per the last 
	///        example. 
	public __gc class Map: public ByteChangeNotify
	{
	public:
		static const int MODE_IGNITION = 1;
		static const int MODE_FUEL = 0;
		static const int HEADER_EVENT_ALL = -2;
		static const int HEADER_EVENT_NONE = -1;
	protected:
		unsigned char thefile __gc[];
		unsigned char map __gc[,];
		///the column headers
		String * vac[];
		///the row headers
		String * rpm[];

		int revscalarloc;
		int pressscalarloc;

		int x,y; //max dimensions
		unsigned int location;//location of 1st byte

		int index;
		int mode;

		bool vtec;

		String* name;

		HeaderCalculatorPair* headerCalculators;

	private:
		///this will blindly place the headers into our vectors.
		virtual void setHeaders(RomRep::MapHeaderChangeArgs* change);
	
	protected:
		void MapHeaderChanged(int xx, int yy, String* realval[]){
			MapHeaderChanged(new MapHeaderChangeArgs(xx,yy,realval));
		}
		void MapHeaderChanged(MapHeaderChangeArgs* args){
			if (OnMapHeaderChanged != NULL)
				OnMapHeaderChanged->Invoke(this, args);
		}
		void MapHeaderEnableChanged(int xx, int yy, bool enabled){
			if (OnMapHeaderEnableChanged != NULL)
				OnMapHeaderEnableChanged->Invoke(this, new MapEnableChangeArgs(xx,yy,enabled));
		}
		void MapChanged(int xx, int yy, int byte){
			if (OnMapChanged != NULL)
				OnMapChanged->Invoke(this, new MapChangeArgs(getCellLocation(xx,yy),xx,yy,byte));
		}
		void RealMapChanged(int xx, int yy, int byte){
			float realval = calculate(xx,yy,byte);
			if (OnRealMapChanged != NULL)
				OnRealMapChanged->Invoke(this, new RealMapChangeArgs(xx,yy,realval));
		}

	public: //public for the headerCalculators
		void SendByteChangeRequest(int loc, unsigned char byte){
			SendByteChangeRequest(new ByteChange(loc,(char)byte));
		}
		void SendByteChangeRequest(ByteChange* ch){
			if(OnByteChangeRequest != NULL)
				OnByteChangeRequest->Invoke(this, new ByteChangeArgs(ch));
		}
		void SendByteChangeRequest(ByteChange* changes[]);

	public:

		Map();
		Map(int x, int y, unsigned int locale, int ind, int mode); //width, height
		//!Normal Constructor
		Map(int x, int y, unsigned int locale, unsigned char fil __gc[], int ind, int mode);

		///Sent when the map wants to change a byte in the rom. It is assumed that 
		///the HondaEcu object containing this map is subscribed to the event. The 
		///special maps call this when a map header is changed.
		[NonSerialized] __event MapByteChangeRequest * OnByteChangeRequest;
		
		///Event fired when this map is Changed in that something called setVal().
		///So really this is just issuing a byte change request to the HondaEcu object.
		[NonSerialized] __event MapChangeHandler * OnMapChanged;
		///This works in conjunction with OnMapChanged event. OnMapChanged will send
		///a byte requst to the HondaEcu object, which will set the byte then notify all
		///of its maps of the byte change (via the byteChangeNotify method). If the byte 
		///change is within the map, the map will fire the OnRealMapChanged event with the real
		///value of the map to signal that a byte change has taken place.
		[NonSerialized] __event RealMapChangeHandler * OnRealMapChanged;

		///Called when a header in the map has been changed. In the resulting 
		///MapHeaderChangeArgs, you will receive both the row or column that this 
		///change took place in and the data (string) to use in that area. For instance
		///say column header 14 is changed to "1.3psi". The column variable would have 
		///14, the row variable would have Map::HEADER_EVENT_NONE, and the data would be a
		///string array with "1.3psi" as its only element. Additionally, it is possible to 
		///receive a Map::HEADER_EVENT_ALL in place of the row or column. The data would then
		///have a string array with all of the headers.
		[NonSerialized] __event MapHeaderChangeHandler * OnMapHeaderChanged;

		///Called when the editable state of a header has changed. i.e. Only the TPS
		///portion of an ITB map is editable and the user changes the xover column. 
		///In that case, OnMapHeaderEnableChanged would be called for each column that 
		///changed from map->tps or vice versa with Map::HEADER_EVENT_NONE as the y
		///component. A situation like that is sort of a rarity, as most of the maps
		///are all or nothing with no changes that will ever happen.
		[NonSerialized] __event MapHeaderEnableChangeHandler * OnMapHeaderEnableChanged;

		//!useless destructor
		virtual ~Map(); //virtual calls destruct when child destructs

		//!Gets the real world values of the map.
		//!\return Returns a managed float array (float[,]) 
		float getMap() __gc[,];

		//!Gets the RealWorld value at a location in the map. Indexed from 0.
		virtual float getVal(int x, int y); 
		//!Sets a value in the map
		virtual RomRep::ByteChange* setVal(int x, int y, float val);

	//	virtual void insertColumn(int column); //column inserted AFTER 'column'
	//	virtual void insertRow(int row); //column inserted AFTER 'row'

		///Takes byte (an integer) and returns real world value
		///
		///This must be overridden so this base class can properly 
		///convert between bytes and real world values.
		virtual float calculate(int x, int y, int intval) = 0;
		///Takes real world value and returns an integer (0-255 byte)
		///
		///This must be overridden so this base class can properly 
		///convert between bytes and real world values.
		virtual int calculate(int x, int y, float mapval) = 0;

		//!populates the map with the byte array
		virtual void populate(unsigned char mapp __gc[,]);

		//!Returns map width
		virtual int getX(){return x;}
		//!Returns map height
		virtual int getY(){return y;}

		///Index is for distinguishing between vtec/non boost/non, etc.
		virtual int getIndex(){ return index; };
		///Mode is for distinguishing between fuel and ign
		virtual int getMode(){ return mode; }

		//!Returns whether the map is vtec (true) or not (false)
		virtual bool isVtec(){return vtec;};

		//!Returns the integer ROM location of the map
		virtual unsigned int getMapLocation(){return location;};
		//!Sets the ROM location of the map
		virtual void setMapLocation(unsigned int loc){location = loc;};
		//!Returns the ROM location of a cell in the map
		virtual unsigned int getCellLocation(int x, int y);
		//!Returns the ROM location of a cell in the map (static)
		static unsigned int getCellLocation(int x, int y, int cols, int loc);
		//!Sets a multiplier (if fuel map; ign maps return null)
		virtual RomRep::ByteChange* setMultiplier(int column, int val) = 0;
		//!Returns the ROM location of the multiplier
		virtual int getMultiplierLocation(int column) = 0;

		//!Returns the Row Scalar ROM location
		virtual int getRowScalarLoc(){return revscalarloc;}; 
		//!Returns the Column scalar ROM location
		virtual int getColumnScalarLoc(){return pressscalarloc;};
		//!Sets the Row Scalar ROM location
		virtual void setRowScalarLoc(int val){revscalarloc = val;};
		//!Sets the Column scalar ROM location 
		virtual void setColumnScalarLoc(int val){pressscalarloc = val;};

		//!Returns the header text for the specified row
		virtual String* getRowHeader(int row){return rpm[row];};
		//!Returns the header text for the specified column
		virtual String* getColumnHeader(int col){return vac[col];};

		//!Returns the name of the map
		virtual String* getName(){return name;}
		//!Sets the map's name
		virtual void setName(String* n){name = n;}

		///For each inherited class, this will see if the byte location
		///applies to their object and then make the appropriate updates.
		virtual void byteChangeNotify(ByteChange* bc);

		virtual void setRowHeaderCalculator(HeaderCalculator* he);
		virtual void setColumnHeaderCalculator(HeaderCalculator* he);
		virtual HeaderCalculator* getRowHeaderCalculator(){return headerCalculators->GetRowCalculator();};
		virtual HeaderCalculator* getColumnHeaderCalculator(){return headerCalculators->GetColumnCalculator();};
		///Returns true if the column or row can be edited. Calls to the HeaderEditors.
		virtual bool canEditHeader(bool column, int columnOrRow);
		///Calls the corresponding function in all of the HeaderEditors.
		virtual void setRowHeader(int row, String* val);
		///Calls the corresponding function in all of the HeaderEditors.
		virtual void setColumnHeader(int col, String* val);

		virtual void mapHeaderChanged(Object* sender, MapHeaderChangeArgs* args);
		void notifyHeaderChanged(Object* sender, MapHeaderChangeArgs* args)
		{mapHeaderChanged(sender, args);};
	};
}