#pragma once

using namespace System;

#include "bytechange.h"
#include "romrepeventargs.h"


namespace RomRep
{
	///Handler for when a mapChange event is fired by one of the maps 
	public __delegate void MapChangeHandler(Object *, MapChangeArgs *);

	public __delegate void MapByteChangeRequest(Object*, ByteChangeArgs *);

	public __delegate void RealMapChangeHandler(Object *, RealMapChangeArgs *);
	public __delegate void MapHeaderChangeHandler(Object *, MapHeaderChangeArgs *);

	//!
	//!Idea behind this: Have a base class that all other maps can be derived from.
	//!Data will be stored in its byte and will be returned as real world value.
	//!
	//!Concrete derived Maps will get passed an integer representation of the map (from
	//!the raw file) then will convert the map and store it in this class' 2D array.
	//!
	[Serializable] public __gc class Map: public ByteChangeNotify
	{
	public:
		static const int MODE_IGNITION = 1;
		static const int MODE_FUEL = 0;
		static const int HEADER_EVENT_ALL = -2;
		static const int HEADER_EVENT_NONE = -1;
	protected:
		unsigned char thefile __gc[];
		unsigned char map __gc[,];
		///the column headers
		String * vac[];
		///the row headers
		String * rpm[];

		int revscalarloc;
		int pressscalarloc;

		int x,y; //max dimensions
		unsigned int location;//location of 1st byte

		int index;
		int mode;

		bool vtec;

		String* name;

		void MapHeaderChanged(int xx, int yy, String* realval[])
		{
			if (OnMapHeaderChanged != NULL)
				OnMapHeaderChanged->Invoke(this, new MapHeaderChangeArgs(xx,yy,realval));
		}

		void MapChanged(int xx, int yy, int byte)
		{
			if (OnMapChanged != NULL)
				OnMapChanged->Invoke(this, new MapChangeArgs(getCellLocation(xx,yy),xx,yy,byte));
		}

		void RealMapChanged(int xx, int yy, int byte)
		{
			float realval = calculate(xx,yy,byte);
			if (OnRealMapChanged != NULL)
				OnRealMapChanged->Invoke(this, new RealMapChangeArgs(xx,yy,realval));
		}

		void SendByteChangeRequest(int loc, unsigned char byte)
		{
			if(OnByteChangeRequest != NULL)
				OnByteChangeRequest->Invoke(this, new ByteChangeArgs(new ByteChange(loc,(char)byte)));
		}

	public:

		Map();
		Map(int x, int y, unsigned int locale, int ind, int mode); //width, height
		//!Normal Constructor
		Map(int x, int y, unsigned int locale, unsigned char fil __gc[], int ind, int mode);

		///Sent when the map wants to change a byte in the rom. It is assumed that 
		///the HondaEcu object containing this map is subscribed to the event. The 
		///special maps call this when a map header is changed.
		[NonSerialized] __event MapByteChangeRequest * OnByteChangeRequest;
		
		///Event fired when this map is Changed in that something called setVal().
		//So really this is just issuing a byte change request to the HondaEcu object.
		[NonSerialized] __event MapChangeHandler * OnMapChanged;
		///This works in conjunction with OnMapChanged event. OnMapChanged will send
		///a byte requst to the HondaEcu object, which will set the byte then notify all
		///of its maps of the byte change (via the byteChangeNotify method). If the byte 
		///change is within the map, the map will fire the OnRealMapChanged event with the real
		///value of the map to signal that a byte change has taken place.
		[NonSerialized] __event RealMapChangeHandler * OnRealMapChanged;

		///Called when a header in the map has been changed. In the resulting 
		///MapHeaderChangeArgs, you will receive both the row or column that this 
		///change took place in and the data (string) to use in that area. For instance
		///say column header 14 is changed to "1.3psi". The column variable would have 
		///14, the row variable would have Map::HEADER_EVENT_NONE, and the data would be a
		///string array with "1.3psi" as its only element. Additionally, it is possible to 
		///receive a Map::HEADER_EVENT_ALL in place of the row or column. The data would then
		///have a string array with all of the headers.
		[NonSerialized] __event MapHeaderChangeHandler * OnMapHeaderChanged;

		//!useless destructor
		virtual ~Map(); //virtual calls destruct when child destructs

		//!Gets the real world values of the map.
		//!\return Returns a managed float array (float[,]) 
		float getMap() __gc[,];

		//!Gets the RealWorld value at a location in the map. Indexed from 0.
		virtual float getVal(int x, int y); 
		//!Sets a value in the map
		virtual RomRep::ByteChange* setVal(int x, int y, float val);

	//	virtual void insertColumn(int column); //column inserted AFTER 'column'
	//	virtual void insertRow(int row); //column inserted AFTER 'row'

		///Takes byte (an integer) and returns real world value
		///
		///This must be overridden so this base class can properly 
		///convert between bytes and real world values.
		virtual float calculate(int x, int y, int intval) = 0;
		///Takes real world value and returns an integer (0-255 byte)
		///
		///This must be overridden so this base class can properly 
		///convert between bytes and real world values.
		virtual int calculate(int x, int y, float mapval) = 0;

		//!populates the map with the byte array
		virtual void populate(unsigned char mapp __gc[,]);

		//!Returns map width
		virtual int getX(){return x;}
		//!Returns map height
		virtual int getY(){return y;}

		///Index is for distinguishing between vtec/non boost/non, etc.
		virtual int getIndex(){ return index; };
		///Mode is for distinguishing between fuel and ign
		virtual int getMode(){ return mode; }

		//!Returns whether the map is vtec (true) or not (false)
		virtual bool isVtec(){return vtec;};

		//!Returns the integer ROM location of the map
		virtual unsigned int getMapLocation(){return location;};
		//!Sets the ROM location of the map
		virtual void setMapLocation(unsigned int loc){location = loc;};
		//!Returns the ROM location of a cell in the map
		virtual unsigned int getCellLocation(int x, int y);
		//!Returns the ROM location of a cell in the map (static)
		static unsigned int getCellLocation(int x, int y, int cols, int loc);
		//!Sets a multiplier (if fuel map; ign maps return null)
		virtual RomRep::ByteChange* setMultiplier(int column, int val) = 0;
		//!Returns the ROM location of the multiplier
		virtual int getMultiplierLocation(int column) = 0;

		//!Returns the Row Scalar ROM location
		virtual int getRowScalarLoc(){return revscalarloc;}; 
		//!Returns the Column scalar ROM location
		virtual int getColumnScalarLoc(){return pressscalarloc;};
		//!Sets the Row Scalar ROM location
		virtual void setRowScalarLoc(int val){revscalarloc = val;};
		//!Sets the Column scalar ROM location 
		virtual void setColumnScalarLoc(int val){pressscalarloc = val;};

		//!Returns the header text for the specified row
		virtual String* getRowHeader(int row){return rpm[row];};
		//!Returns the header text for the specified column
		virtual String* getColumnHeader(int col){return vac[col];};

		//!Returns the name of the map
		virtual String* getName(){return name;}
		//!Sets the map's name
		virtual void setName(String* n){name = n;}

		///For each inherited class, this will see if the byte location
		///applies to their object and then make the appropriate updates.
		virtual void byteChangeNotify(ByteChange* bc);

		///This returns true if the column or row can be edited. 
		virtual bool canEditHeader(bool column, int columnOrRow)
		{ return false; }

		///These set functions should take the string, use a regex 
		///and try to extract the data. If possible, they will calculate
		///the byte(s) to be changed and do a send byte request to the 
		///parent. 
		///
		///This function sets the header text for the specified row.
		virtual void setRowHeader(int row, String* val){
		//	rpm[row] = val;
		//	MapHeaderChanged(-1,row,val);
		};
		///These set functions should take the string, use a regex 
		///and try to extract the data. If possible, they will calculate
		///the byte(s) to be changed and do a send byte request to the 
		///parent. 
		///
		///This function sets the header text for the specified column
		virtual void setColumnHeader(int col, String* val){
		//	vac[col] = val;
		//	MapHeaderChanged(col,-1,val);
		};
	};
}